// SPDX-License-Identifier: GPL-2.0+
/*
 * Copyright 2018 NXP
 */

#define DEBUG
#undef CONFIG_LOGLEVEL
#define CONFIG_LOGLEVEL 8


#include <common.h>
#include <miiphy.h>
#include <netdev.h>
#include <asm/mach-imx/iomux-v3.h>
#include <asm-generic/gpio.h>
#include <asm/arch/imx8mm_pins.h>
#include <asm/arch/clock.h>
#include <asm/arch/sys_proto.h>
#include <asm/mach-imx/gpio.h>
#include <asm/mach-imx/mxc_i2c.h>
#include <i2c.h>
#include <asm/io.h>
#include "../common/tcpc.h"
#include <usb.h>

#include <mipi_dsi.h>
#include <dsi_host.h>
#include <imx_sip.h>
#include <linux/arm-smccc.h>

DECLARE_GLOBAL_DATA_PTR;

#define UART_PAD_CTRL	(PAD_CTL_DSE6 | PAD_CTL_FSEL1)
#define WDOG_PAD_CTRL	(PAD_CTL_DSE6 | PAD_CTL_ODE | PAD_CTL_PUE | PAD_CTL_PE)
#define USB_PAD_CTRL	(PAD_CTL_DSE6)

static iomux_v3_cfg_t const uart_pads[] = {
	/*IMX8MM_PAD_UART2_RXD_UART2_RX | MUX_PAD_CTRL(UART_PAD_CTRL),
	IMX8MM_PAD_UART2_TXD_UART2_TX | MUX_PAD_CTRL(UART_PAD_CTRL),*/
	IMX8MM_PAD_SAI2_RXC_UART1_RX | MUX_PAD_CTRL(UART_PAD_CTRL),
	IMX8MM_PAD_SAI2_RXFS_UART1_TX | MUX_PAD_CTRL(UART_PAD_CTRL),
};

static iomux_v3_cfg_t const wdog_pads[] = {
	IMX8MM_PAD_GPIO1_IO02_WDOG1_WDOG_B  | MUX_PAD_CTRL(WDOG_PAD_CTRL),
};

static iomux_v3_cfg_t const usbotg1_pads[] = {
	IMX8MM_PAD_GPIO1_IO10_USB1_OTG_ID,
};

static iomux_v3_cfg_t const usbotg2_pads[] = {
	IMX8MM_PAD_GPIO1_IO11_USB2_OTG_ID,
};


#ifdef CONFIG_NAND_MXS
#ifdef CONFIG_SPL_BUILD
#define NAND_PAD_CTRL	(PAD_CTL_DSE6 | PAD_CTL_FSEL2 | PAD_CTL_HYS)
#define NAND_PAD_READY0_CTRL (PAD_CTL_DSE6 | PAD_CTL_FSEL2 | PAD_CTL_PUE)
static iomux_v3_cfg_t const gpmi_pads[] = {
	IMX8MM_PAD_NAND_ALE_RAWNAND_ALE | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_CE0_B_RAWNAND_CE0_B | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_CE1_B_RAWNAND_CE1_B | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_CLE_RAWNAND_CLE | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA00_RAWNAND_DATA00 | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA01_RAWNAND_DATA01 | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA02_RAWNAND_DATA02 | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA03_RAWNAND_DATA03 | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA04_RAWNAND_DATA04 | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA05_RAWNAND_DATA05	| MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA06_RAWNAND_DATA06	| MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA07_RAWNAND_DATA07	| MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_RE_B_RAWNAND_RE_B | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_READY_B_RAWNAND_READY_B | MUX_PAD_CTRL(NAND_PAD_READY0_CTRL),
	IMX8MM_PAD_NAND_WE_B_RAWNAND_WE_B | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_WP_B_RAWNAND_WP_B | MUX_PAD_CTRL(NAND_PAD_CTRL),
};
#endif

static void setup_gpmi_nand(void)
{
#ifdef CONFIG_SPL_BUILD
	imx_iomux_v3_setup_multiple_pads(gpmi_pads, ARRAY_SIZE(gpmi_pads));
#endif

	init_nand_clk();
}
#endif


#define GPIO_PAD_CTRL	(PAD_CTL_DSE6 | PAD_CTL_PE | PAD_CTL_FSEL2)
/*order: Javelin2Dev_i.MX8Mmini_AssignedPinList_20220216.xlsx*/
static iomux_v3_cfg_t const gpio_pads[] = {
	IMX8MM_PAD_GPIO1_IO11_GPIO1_IO11  | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO03_GPIO1_IO3   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO04_GPIO1_IO4   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI1_RXC_GPIO4_IO1     | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO07_GPIO1_IO7   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO08_GPIO1_IO8   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO09_GPIO1_IO9   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO10_GPIO1_IO10  | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO12_GPIO1_IO12  | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_CLK_GPIO2_IO0      | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_CMD_GPIO2_IO1      | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA0_GPIO2_IO2    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA1_GPIO2_IO3    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA2_GPIO2_IO4    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA3_GPIO2_IO5    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA4_GPIO2_IO6    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA5_GPIO2_IO7    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA6_GPIO2_IO8    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA7_GPIO2_IO9    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_RESET_B_GPIO2_IO10 | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_STROBE_GPIO2_IO11  | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_CD_B_GPIO2_IO12    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_CLK_GPIO2_IO13     | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_CMD_GPIO2_IO14     | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA0_GPIO2_IO15   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA1_GPIO2_IO16   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA2_GPIO2_IO17   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA3_GPIO2_IO18   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_RESET_B_GPIO2_IO19 | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_WP_GPIO2_IO20      | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXFS_GPIO3_IO19   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXC_GPIO3_IO20    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXD0_GPIO3_IO21   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXD1_GPIO3_IO22   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXD2_GPIO3_IO23   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXD3_GPIO3_IO24   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI1_RXD4_GPIO4_IO6    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	/*IMX8MM_PAD_SAI1_RXD6_GPIO4_IO8    | MUX_PAD_CTRL(GPIO_PAD_CTRL),*/ /*controlled by driver*/
	IMX8MM_PAD_SAI1_RXD7_GPIO4_IO9    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI1_TXD5_GPIO4_IO17   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI1_TXD6_GPIO4_IO18   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI1_TXD7_GPIO4_IO19   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI2_RXD0_GPIO4_IO23   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI2_TXFS_GPIO4_IO24   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI2_TXC_GPIO4_IO25    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI2_TXD0_GPIO4_IO26   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI3_MCLK_GPIO5_IO2    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI3_RXFS_GPIO4_IO28   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI3_TXD_GPIO5_IO1     | MUX_PAD_CTRL(GPIO_PAD_CTRL),
};

/*order: Javelin2Dev_i.MX8Mmini_AssignedPinList_20220216.xlsx*/
#define STBY_REBOOT_GPIO			IMX_GPIO_NR(1, 11)
#define PMIC_XINT_GPIO				IMX_GPIO_NR(1, 3)
#define FE_CHG_STATUS_GPIO			IMX_GPIO_NR(1, 4)
#define FE_CHG_STATUS_M4_GPIO		IMX_GPIO_NR(4, 0)
#define FE_UPDATE_SW_GPIO			IMX_GPIO_NR(1, 7)
#define BOOTUP_NOTE_GPIO			IMX_GPIO_NR(1, 8)
#define PWR_RESET_GPIO				IMX_GPIO_NR(1, 9)
#define X_BE_RESET_GPIO				IMX_GPIO_NR(1, 10)
#define EDLC_CHANGE_START_GPIO		IMX_GPIO_NR(1, 12)
#define SPI_SEL_GPIO				IMX_GPIO_NR(2, 0)
#define PWR_GOOD_PIF_GPIO			IMX_GPIO_NR(2, 1)
#define RST_PIF_N_GPIO				IMX_GPIO_NR(2, 2)
#define SPI_NR_SEL_GPIO				IMX_GPIO_NR(2, 3)
#define BE_POWER_ON2_GPIO			IMX_GPIO_NR(2, 4)
#define BE_POWER_ON1_GPIO			IMX_GPIO_NR(2, 5)
#define PWR_GOOD_DIF_GPIO			IMX_GPIO_NR(2, 6)
#define RESERVED0_GPIO				IMX_GPIO_NR(2, 7)
#define IRQ_DIF_N_GPIO				IMX_GPIO_NR(2, 8)
#define RST_DIF_N_GPIO				IMX_GPIO_NR(2, 9)
#define STBY_DSI_BRG_GPIO			IMX_GPIO_NR(2, 10)
#define BE_POWER_ON0_GPIO			IMX_GPIO_NR(2, 11)
#define PANEL_DRV_MUTE_GPIO			IMX_GPIO_NR(2, 12)
#define IRQ_PIF_N_GPIO				IMX_GPIO_NR(2, 13)
#define PCIE_NRST_GPIO				IMX_GPIO_NR(2, 14)
#define LED_SEL_GPIO				IMX_GPIO_NR(2, 15)
#define IMX_STBY_LED_GPIO			IMX_GPIO_NR(2, 16)
#define CLKGEN_UPDATE_GPIO			IMX_GPIO_NR(2, 17)
#define CLKGEN_ROM_SEL_GPIO			IMX_GPIO_NR(2, 18)
#define IMX_POWER_LED_GPIO			IMX_GPIO_NR(2, 19)
#define FE_PANEL_DRV_MUTE_IN_GPIO	IMX_GPIO_NR(2, 20)
#define EDID_SEL1_GPIO				IMX_GPIO_NR(3, 19)
#define EDID_SEL2_GPIO				IMX_GPIO_NR(3, 20)
#define HDMI1_HPD_GPIO				IMX_GPIO_NR(3, 21)
#define EDID_WP_GPIO				IMX_GPIO_NR(3, 22)
#define HDMI2_HPD_GPIO				IMX_GPIO_NR(3, 23)
#define SELF_RESET_GPIO				IMX_GPIO_NR(3, 24)
#define RST_DSI_LVDS_BRG_GPIO		IMX_GPIO_NR(4, 6)
/*#define ETHER_PHY_RST_GPIO			IMX_GPIO_NR(4, 8)*/ /*controlled by driver*/
#define BE_WDT_GPIO					IMX_GPIO_NR(4, 9)
#define FAN_PROT_GPIO				IMX_GPIO_NR(4, 17)
#define AU_MUTE_A_GPIO				IMX_GPIO_NR(4, 18)
#define PWR_ON_VBUS_IMX_GPIO		IMX_GPIO_NR(4, 19)
#define IMX_LAMP_LED_GPIO			IMX_GPIO_NR(4, 23)
#define AC_OFF_DET_GPIO				IMX_GPIO_NR(4, 24)
#define HW_PROT_DET_GPIO			IMX_GPIO_NR(4, 25)
#define OCP_VBUX_IMX_GPIO			IMX_GPIO_NR(4, 26)
#define BE_UPDATE_SW_GPIO			IMX_GPIO_NR(5, 2)

#define CEC_IN_GPIO					IMX_GPIO_NR(4, 28)
#define CEC_OUT_GPIO				IMX_GPIO_NR(5, 1)

static void setup_gpio(void)
{

	debug("start GPIO init.\n");

	imx_iomux_v3_setup_multiple_pads(gpio_pads, ARRAY_SIZE(gpio_pads));

#define GPIO_REQUEST_IN(x) do{gpio_request(x, #x); gpio_direction_input(x);}while(0)
#define GPIO_REQUEST_OUT(x, y) do{gpio_request(x, #x); gpio_direction_output(x, y);}while(0)

	GPIO_REQUEST_OUT(STBY_REBOOT_GPIO, 0);
	GPIO_REQUEST_IN(PMIC_XINT_GPIO);
	GPIO_REQUEST_IN(FE_CHG_STATUS_GPIO);
	GPIO_REQUEST_IN(FE_CHG_STATUS_M4_GPIO);
	GPIO_REQUEST_OUT(FE_UPDATE_SW_GPIO, 0);
	GPIO_REQUEST_IN(BOOTUP_NOTE_GPIO);
	GPIO_REQUEST_OUT(PWR_RESET_GPIO, 0);
	GPIO_REQUEST_OUT(X_BE_RESET_GPIO, 1);
	GPIO_REQUEST_OUT(EDLC_CHANGE_START_GPIO, 0);
	GPIO_REQUEST_OUT(SPI_SEL_GPIO, 0);
	GPIO_REQUEST_IN(PWR_GOOD_PIF_GPIO);
	GPIO_REQUEST_OUT(RST_PIF_N_GPIO, 1);
	GPIO_REQUEST_OUT(SPI_NR_SEL_GPIO, 0);
	GPIO_REQUEST_OUT(BE_POWER_ON2_GPIO, 0);
	GPIO_REQUEST_OUT(BE_POWER_ON1_GPIO, 0);
	GPIO_REQUEST_IN(PWR_GOOD_DIF_GPIO);
	GPIO_REQUEST_OUT(RESERVED0_GPIO, 0);
	GPIO_REQUEST_IN(IRQ_DIF_N_GPIO);
	GPIO_REQUEST_OUT(RST_DIF_N_GPIO, 0);
	GPIO_REQUEST_OUT(STBY_DSI_BRG_GPIO, 0);
	GPIO_REQUEST_OUT(BE_POWER_ON0_GPIO, 0);
	GPIO_REQUEST_OUT(PANEL_DRV_MUTE_GPIO, 0);
	GPIO_REQUEST_IN(IRQ_PIF_N_GPIO);
	GPIO_REQUEST_OUT(PCIE_NRST_GPIO, 0);
	GPIO_REQUEST_OUT(LED_SEL_GPIO, 1);
	GPIO_REQUEST_OUT(IMX_STBY_LED_GPIO, 0);
	GPIO_REQUEST_OUT(CLKGEN_UPDATE_GPIO, 0);
	GPIO_REQUEST_OUT(CLKGEN_ROM_SEL_GPIO,0);
	GPIO_REQUEST_OUT(IMX_POWER_LED_GPIO, 0);
	GPIO_REQUEST_IN(FE_PANEL_DRV_MUTE_IN_GPIO);
	GPIO_REQUEST_OUT(EDID_SEL1_GPIO, 1);
	GPIO_REQUEST_OUT(EDID_SEL2_GPIO, 1);
	GPIO_REQUEST_OUT(HDMI1_HPD_GPIO,0);
	GPIO_REQUEST_OUT(EDID_WP_GPIO, 1);
	GPIO_REQUEST_OUT(HDMI2_HPD_GPIO, 0);
	GPIO_REQUEST_OUT(SELF_RESET_GPIO, 0);
	GPIO_REQUEST_OUT(RST_DSI_LVDS_BRG_GPIO, 0);
	/*GPIO_REQUEST_OUT(ETHER_PHY_RST_GPIO, 0);*/ /*controlled by driver*/
	GPIO_REQUEST_IN(BE_WDT_GPIO);
	GPIO_REQUEST_IN(FAN_PROT_GPIO);
	GPIO_REQUEST_OUT(AU_MUTE_A_GPIO, 1);
	GPIO_REQUEST_OUT(PWR_ON_VBUS_IMX_GPIO, 1);
	GPIO_REQUEST_OUT(IMX_LAMP_LED_GPIO, 0);
	GPIO_REQUEST_IN(AC_OFF_DET_GPIO);
	GPIO_REQUEST_IN(HW_PROT_DET_GPIO);
	GPIO_REQUEST_IN(OCP_VBUX_IMX_GPIO);
	GPIO_REQUEST_OUT(BE_UPDATE_SW_GPIO, 0);
	GPIO_REQUEST_IN(CEC_IN_GPIO);
	GPIO_REQUEST_OUT(CEC_OUT_GPIO, 0);

	debug("end GPIO init.\n");

/*
	GPIO-HOG can't work on imx8mm?
	always "Header Tag is not an IMX image"
*/
/*
	struct gpio_desc *desc;
	int ret, i;
	static char* pin_names[] = {
		"fe_chg_status",
		"fe_update_sw",
		"bootup_note",
		"pwr_restart",
		"x_be_reset",
		"stby_reboot",
		"edlc_charge_start",
		"spi_sel",
		"pwr_good_pif",
		"rst_pif_n",
		"spi_nr_sel",
		"be_power_on2",
		"be_power_on1",
		"pwr_good_dif",
		"reserved_2_7",
		"irq_dif_n",
		"rst_dif_n",
		"be_power_on0",
		"panel_drv_mute",
		"irq_pif_n",
		"pcie_nrst",
		"led_sel",
		"imx_stby_led",
		"clkgen_update",
		"clkgen_rom_sel",
		"imx_power_led",
		"fe_panel_drv_mute_in",
		"edid_sel1",
		"edid_sel2",
		"hdmi1_hpd",
		"edid_wp",
		"hdmi2_hpd",
		"self_reset",
		"fe_chg_status_m4",
		"rst_dsi_lvds_brg",
		"be_wdt",
		"fan_prot",
		"au_mute_a",
		"pwr_on_vbus_imx",
		"imx_lamp_led",
		"ac_off_det",
		"ocp_vbus_imx",
		"hw_prot_det",
		"cec_in",
		"cec_out",
		"be_update_sw",
	};
	printf("start GPIO init.\n");
	ret = gpio_hog_probe_all();
	if(ret){
		printf("gpio_hog_probe_all error:%d\n", ret);
	}
	for(i = 0; i < ARRAY_SIZE(pin_names); ++i){
		ret = gpio_hog_lookup_name(pin_names[i], &desc);
		if(ret){
			printf("gpio-hog %d:%s not found.\n", i, pin_names[i]);
		}
	}
	printf("end GPIO init %d.\n", i);
*/
}

int board_early_init_f(void)
{
	struct wdog_regs *wdog = (struct wdog_regs *)WDOG1_BASE_ADDR;

	imx_iomux_v3_setup_multiple_pads(wdog_pads, ARRAY_SIZE(wdog_pads));

	set_wdog_reset(wdog);

	imx_iomux_v3_setup_multiple_pads(uart_pads, ARRAY_SIZE(uart_pads));
	/*__raw_writel(3, IOMUXC_BASE_ADDR + 0x4f4 );*/ /*as IOMUXC_UART1_RXD_SELECT_INPUT*/

	init_uart_clk(0);

#ifdef CONFIG_NAND_MXS
	setup_gpmi_nand(); /* SPL will call the board_early_init_f */
#endif

	setup_gpio();

	return 0;
}

#if IS_ENABLED(CONFIG_FEC_MXC)
static int setup_fec(void)
{
	debug("stella2 setup_fec() start\n");

	struct iomuxc_gpr_base_regs *gpr =
		(struct iomuxc_gpr_base_regs *)IOMUXC_GPR_BASE_ADDR;




	/* Use 50MHz?*/
	setbits_le32(&gpr->gpr[1],
			IOMUXC_GPR_GPR1_GPR_ENET1_TX_CLK_SEL_MASK);
	debug("stella2 setup_fec() 25MHz end\n");
	return set_clk_enet(ENET_25MHZ);

	/* Use 125M anatop REF_CLK1 for ENET1, not from external */
/*	clrsetbits_le32(&gpr->gpr[1],
			IOMUXC_GPR_GPR1_GPR_ENET1_TX_CLK_SEL_MASK, 0);

	debug("stella2 setup_fec() 125MHz end\n");

	return set_clk_enet(ENET_125MHZ);*/
}

int board_phy_config(struct phy_device *phydev)
{
	debug("stella2 board_phy_config() start with %p\n", phydev);

	/* enable rgmii rxc skew and phy mode select to RGMII copper */
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1d, 0x1f);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1e, 0x8);

	phy_write(phydev, MDIO_DEVAD_NONE, 0x1d, 0x00);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1e, 0x82ee);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1d, 0x05);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1e, 0x100);

	if (phydev->drv->config)
		phydev->drv->config(phydev);

	debug("stella2 board_phy_config() end with %p\n", (phydev->drv->config));

	return 0;
}
#endif

#ifdef CONFIG_USB_TCPC
struct tcpc_port port1;
struct tcpc_port port2;

static int setup_pd_switch(uint8_t i2c_bus, uint8_t addr)
{
	struct udevice *bus;
	struct udevice *i2c_dev = NULL;
	int ret;
	uint8_t valb;

	ret = uclass_get_device_by_seq(UCLASS_I2C, i2c_bus, &bus);
	if (ret) {
		printf("%s: Can't find bus\n", __func__);
		return -EINVAL;
	}

	ret = dm_i2c_probe(bus, addr, 0, &i2c_dev);
	if (ret) {
		printf("%s: Can't find device id=0x%x\n",
			__func__, addr);
		return -ENODEV;
	}

	ret = dm_i2c_read(i2c_dev, 0xB, &valb, 1);
	if (ret) {
		printf("%s dm_i2c_read failed, err %d\n", __func__, ret);
		return -EIO;
	}
	valb |= 0x4; /* Set DB_EXIT to exit dead battery mode */
	ret = dm_i2c_write(i2c_dev, 0xB, (const uint8_t *)&valb, 1);
	if (ret) {
		printf("%s dm_i2c_write failed, err %d\n", __func__, ret);
		return -EIO;
	}

	/* Set OVP threshold to 23V */
	valb = 0x6;
	ret = dm_i2c_write(i2c_dev, 0x8, (const uint8_t *)&valb, 1);
	if (ret) {
		printf("%s dm_i2c_write failed, err %d\n", __func__, ret);
		return -EIO;
	}

	return 0;
}

int pd_switch_snk_enable(struct tcpc_port *port)
{
	if (port == &port1) {
		debug("Setup pd switch on port 1\n");
		return setup_pd_switch(1, 0x72);
	} else if (port == &port2) {
		debug("Setup pd switch on port 2\n");
		return setup_pd_switch(1, 0x73);
	} else
		return -EINVAL;
}

struct tcpc_port_config port1_config = {
	.i2c_bus = 1, /*i2c2*/
	.addr = 0x50,
	.port_type = TYPEC_PORT_UFP,
	.max_snk_mv = 5000,
	.max_snk_ma = 3000,
	.max_snk_mw = 40000,
	.op_snk_mv = 9000,
	.switch_setup_func = &pd_switch_snk_enable,
};

struct tcpc_port_config port2_config = {
	.i2c_bus = 1, /*i2c2*/
	.addr = 0x52,
	.port_type = TYPEC_PORT_UFP,
	.max_snk_mv = 9000,
	.max_snk_ma = 3000,
	.max_snk_mw = 40000,
	.op_snk_mv = 9000,
	.switch_setup_func = &pd_switch_snk_enable,
};

static int setup_typec(void)
{
	int ret;

	debug("tcpc_init port 2\n");
	ret = tcpc_init(&port2, port2_config, NULL);
	if (ret) {
		printf("%s: tcpc port2 init failed, err=%d\n",
		       __func__, ret);
	} else if (tcpc_pd_sink_check_charging(&port2)) {
		/* Disable PD for USB1, since USB2 has priority */
		port1_config.disable_pd = true;
		printf("Power supply on USB2\n");
	}

	debug("tcpc_init port 1\n");
	ret = tcpc_init(&port1, port1_config, NULL);
	if (ret) {
		printf("%s: tcpc port1 init failed, err=%d\n",
		       __func__, ret);
	} else {
		if (!port1_config.disable_pd)
			printf("Power supply on USB1\n");
		return ret;
	}

	return ret;
}

#endif /* #ifdef CONFIG_USB_TCPC */

static int is_usb_initialized = 0;

int board_usb_init(int index, enum usb_init_type init)
{
	int ret = 0;
#ifdef CONFIG_USB_TCPC
	struct tcpc_port *port_ptr;
#endif /* #ifdef CONFIG_USB_TCPC */

	debug("board_usb_init %d, type %d\n", index, init);
	printf("board_usb_init %d, type %d\n", index, init);

	imx8m_usb_power(index, true);

	switch(index){
	case 0:
		imx_iomux_v3_setup_multiple_pads(usbotg1_pads, ARRAY_SIZE(usbotg1_pads));
		break;
	case 1:
		imx_iomux_v3_setup_multiple_pads(usbotg2_pads, ARRAY_SIZE(usbotg2_pads));
		break;
	default:
		printf("No USB %d\n", index);
		break;
	}

#ifdef CONFIG_USB_TCPC
	if (index == 0)
		port_ptr = &port1;
	else
		port_ptr = &port2;

	if (init == USB_INIT_HOST)
		tcpc_setup_dfp_mode(port_ptr);
	else
		tcpc_setup_ufp_mode(port_ptr);
#endif /* #ifdef CONFIG_USB_TCPC */

	is_usb_initialized = 1;
	return ret;
}

int board_usb_cleanup(int index, enum usb_init_type init)
{
	int ret = 0;

	debug("board_usb_cleanup %d, type %d\n", index, init);

#ifdef CONFIG_USB_TCPC
	if (init == USB_INIT_HOST) {
		if (index == 0)
			ret = tcpc_disable_src_vbus(&port1);
		else
			ret = tcpc_disable_src_vbus(&port2);
	}
#endif /* #ifdef CONFIG_USB_TCPC */

	imx8m_usb_power(index, false);
	return ret;
}

#ifdef CONFIG_USB_TCPC
int board_ehci_usb_phy_mode(struct udevice *dev)
{
	int ret = 0;
	enum typec_cc_polarity pol;
	enum typec_cc_state state;
	struct tcpc_port *port_ptr;

	if (dev->req_seq == 0)
		port_ptr = &port1;
	else
		port_ptr = &port2;

	tcpc_setup_ufp_mode(port_ptr);

	ret = tcpc_get_cc_status(port_ptr, &pol, &state);
	if (!ret) {
		if (state == TYPEC_STATE_SRC_RD_RA || state == TYPEC_STATE_SRC_RD)
			return USB_INIT_HOST;
	}

	return USB_INIT_DEVICE;
}
#endif /* #ifdef CONFIG_USB_TCPC */

#define FSL_SIP_GPC			0xC2000000
#define FSL_SIP_CONFIG_GPC_PM_DOMAIN	0x3
#define DISPMIX				9
#define MIPI				10

int board_init(void)
{
	struct arm_smccc_res res;

	debug("stella2 board_init() start\n");
#ifdef CONFIG_USB_TCPC
	setup_typec();
#endif

	if (IS_ENABLED(CONFIG_FEC_MXC))
		setup_fec();

	arm_smccc_smc(IMX_SIP_GPC, IMX_SIP_GPC_PM_DOMAIN,
		      DISPMIX, true, 0, 0, 0, 0, &res);
	arm_smccc_smc(IMX_SIP_GPC, IMX_SIP_GPC_PM_DOMAIN,
		      MIPI, true, 0, 0, 0, 0, &res);

	debug("stella2 board_init() end\n");
	return 0;
}

int board_late_init(void)
{
	debug("stella2 board_late_init() start\n");

#ifdef CONFIG_ENV_IS_IN_MMC
	board_late_mmc_env_init();
#endif

#ifdef CONFIG_ENV_VARS_UBOOT_RUNTIME_CONFIG
	env_set("board_name", "STELLA2 EVA");
	env_set("board_rev", "iMX8MM");
#endif

	debug("stella2 board_late_init() end, is_usb_initialized=%d\n", is_usb_initialized);
	return 0;
}

#ifdef CONFIG_FSL_FASTBOOT
#ifdef CONFIG_ANDROID_RECOVERY
int is_recovery_key_pressing(void)
{
	return 0; /*TODO*/
}
#endif /*CONFIG_ANDROID_RECOVERY*/
#endif /*CONFIG_FSL_FASTBOOT*/

#ifdef CONFIG_ANDROID_SUPPORT
bool is_power_key_pressed(void) {
	return (bool)(!!(readl(SNVS_HPSR) & (0x1 << 6)));
}
#endif

