// SPDX-License-Identifier: GPL-2.0+
/*
 * Copyright 2018 NXP
 */

#define DEBUG
#undef CONFIG_LOGLEVEL
#define CONFIG_LOGLEVEL 8


#include <common.h>
#include <miiphy.h>
#include <netdev.h>
#include <asm/mach-imx/iomux-v3.h>
#include <asm-generic/gpio.h>
#include <asm/arch/imx8mm_pins.h>
#include <asm/arch/clock.h>
#include <asm/arch/sys_proto.h>
#include <asm/mach-imx/gpio.h>
#include <asm/mach-imx/mxc_i2c.h>
#include <i2c.h>
#include <asm/io.h>
#include "../common/tcpc.h"
#include <usb.h>

#include <mipi_dsi.h>
#include <dsi_host.h>
#include <imx_sip.h>
#include <linux/arm-smccc.h>
#include <linux/delay.h>

DECLARE_GLOBAL_DATA_PTR;

#define UART_PAD_CTRL	(PAD_CTL_DSE6 | PAD_CTL_FSEL1)
#define WDOG_PAD_CTRL	(PAD_CTL_DSE6 | PAD_CTL_ODE | PAD_CTL_PUE | PAD_CTL_PE)
#define USB_PAD_CTRL	(PAD_CTL_DSE6)

static iomux_v3_cfg_t const uart_pads[] = {
	/*IMX8MM_PAD_UART2_RXD_UART2_RX | MUX_PAD_CTRL(UART_PAD_CTRL),
	IMX8MM_PAD_UART2_TXD_UART2_TX | MUX_PAD_CTRL(UART_PAD_CTRL),*/
	IMX8MM_PAD_SAI2_RXC_UART1_RX | MUX_PAD_CTRL(UART_PAD_CTRL),
	IMX8MM_PAD_SAI2_RXFS_UART1_TX | MUX_PAD_CTRL(UART_PAD_CTRL),
};

static iomux_v3_cfg_t const wdog_pads[] = {
	IMX8MM_PAD_GPIO1_IO02_WDOG1_WDOG_B  | MUX_PAD_CTRL(WDOG_PAD_CTRL),
};

static iomux_v3_cfg_t const usbotg1_pads[] = {
	IMX8MM_PAD_GPIO1_IO10_USB1_OTG_ID,
};

static iomux_v3_cfg_t const usbotg2_pads[] = {
	IMX8MM_PAD_GPIO1_IO11_USB2_OTG_ID,
};


#ifdef CONFIG_NAND_MXS
#ifdef CONFIG_SPL_BUILD
#define NAND_PAD_CTRL	(PAD_CTL_DSE6 | PAD_CTL_FSEL2 | PAD_CTL_HYS)
#define NAND_PAD_READY0_CTRL (PAD_CTL_DSE6 | PAD_CTL_FSEL2 | PAD_CTL_PUE)
static iomux_v3_cfg_t const gpmi_pads[] = {
	IMX8MM_PAD_NAND_ALE_RAWNAND_ALE | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_CE0_B_RAWNAND_CE0_B | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_CE1_B_RAWNAND_CE1_B | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_CLE_RAWNAND_CLE | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA00_RAWNAND_DATA00 | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA01_RAWNAND_DATA01 | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA02_RAWNAND_DATA02 | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA03_RAWNAND_DATA03 | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA04_RAWNAND_DATA04 | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA05_RAWNAND_DATA05	| MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA06_RAWNAND_DATA06	| MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA07_RAWNAND_DATA07	| MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_RE_B_RAWNAND_RE_B | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_READY_B_RAWNAND_READY_B | MUX_PAD_CTRL(NAND_PAD_READY0_CTRL),
	IMX8MM_PAD_NAND_WE_B_RAWNAND_WE_B | MUX_PAD_CTRL(NAND_PAD_CTRL),
	IMX8MM_PAD_NAND_WP_B_RAWNAND_WP_B | MUX_PAD_CTRL(NAND_PAD_CTRL),
};
#endif

static void setup_gpmi_nand(void)
{
#ifdef CONFIG_SPL_BUILD
	imx_iomux_v3_setup_multiple_pads(gpmi_pads, ARRAY_SIZE(gpmi_pads));
#endif

	init_nand_clk();
}
#endif


#define GPIO_PAD_CTRL	(PAD_CTL_DSE6 | PAD_CTL_PE | PAD_CTL_FSEL2)
/*order: Javelin2Dev_i.MX8Mmini_AssignedPinList_20220216.xlsx*/
static iomux_v3_cfg_t const gpio_pads[] = {
	IMX8MM_PAD_GPIO1_IO11_GPIO1_IO11  | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO03_GPIO1_IO3   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO04_GPIO1_IO4   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI1_RXC_GPIO4_IO1     | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO07_GPIO1_IO7   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO08_GPIO1_IO8   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO09_GPIO1_IO9   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO10_GPIO1_IO10  | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO12_GPIO1_IO12  | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_GPIO1_IO15_GPIO1_IO15  | MUX_PAD_CTRL(GPIO_PAD_CTRL), /*change from PWM4 to GPIO1_15 for MT9810 NotifyFromImx*/
	IMX8MM_PAD_SD1_CLK_GPIO2_IO0      | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_CMD_GPIO2_IO1      | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA0_GPIO2_IO2    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA1_GPIO2_IO3    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA2_GPIO2_IO4    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA3_GPIO2_IO5    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA4_GPIO2_IO6    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA5_GPIO2_IO7    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA6_GPIO2_IO8    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_DATA7_GPIO2_IO9    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_RESET_B_GPIO2_IO10 | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD1_STROBE_GPIO2_IO11  | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_CD_B_GPIO2_IO12    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_CLK_GPIO2_IO13     | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_CMD_GPIO2_IO14     | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA0_GPIO2_IO15   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA1_GPIO2_IO16   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA2_GPIO2_IO17   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA3_GPIO2_IO18   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_RESET_B_GPIO2_IO19 | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SD2_WP_GPIO2_IO20      | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXFS_GPIO3_IO19   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXC_GPIO3_IO20    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXD0_GPIO3_IO21   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXD1_GPIO3_IO22   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXD2_GPIO3_IO23   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI5_RXD3_GPIO3_IO24   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI1_RXD4_GPIO4_IO6    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	/*IMX8MM_PAD_SAI1_RXD6_GPIO4_IO8    | MUX_PAD_CTRL(GPIO_PAD_CTRL),*/ /*controlled by driver*/
	IMX8MM_PAD_SAI1_RXD7_GPIO4_IO9    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI1_TXD5_GPIO4_IO17   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI1_TXD6_GPIO4_IO18   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI1_TXD7_GPIO4_IO19   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI2_RXD0_GPIO4_IO23   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI2_TXFS_GPIO4_IO24   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI2_TXC_GPIO4_IO25    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI2_TXD0_GPIO4_IO26   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI3_MCLK_GPIO5_IO2    | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI3_RXFS_GPIO4_IO28   | MUX_PAD_CTRL(GPIO_PAD_CTRL),
	IMX8MM_PAD_SAI3_TXD_GPIO5_IO1     | MUX_PAD_CTRL(GPIO_PAD_CTRL),
};

/*order: Javelin2Dev_i.MX8Mmini_AssignedPinList_20220216.xlsx*/
#define USB_B_C_SEL_GPIO			IMX_GPIO_NR(1, 11)
#define PMIC_XINT_GPIO				IMX_GPIO_NR(1, 3)
#define FE_CHG_STATUS_GPIO			IMX_GPIO_NR(1, 4)
#define FE_RESUME_IMX_GPIO			IMX_GPIO_NR(4, 0)
#define FE_UPDATE_SW_GPIO			IMX_GPIO_NR(1, 7)
#define BOOT_DONE_USB_C_GPIO		IMX_GPIO_NR(1, 8)
#define PWR_RESET_GPIO				IMX_GPIO_NR(1, 9)
#define X_BE_RESET_GPIO				IMX_GPIO_NR(1, 10)
#define LED_ILLUMI_POW_EN_GPIO		IMX_GPIO_NR(1, 12)
#define NOTIFY_FROM_IMX_GPIO		IMX_GPIO_NR(1, 15)
#define SPI_SEL_GPIO				IMX_GPIO_NR(2, 0)
#define INT_MIPI_TO_EDP_GPIO		IMX_GPIO_NR(2, 1)
#define USB_UPDATE_EN_GPIO			IMX_GPIO_NR(2, 2)
#define SPI_NR_SEL_GPIO				IMX_GPIO_NR(2, 3)
#define BE_POWER_ON2_GPIO			IMX_GPIO_NR(2, 4)
#define BE_POWER_ON1_GPIO			IMX_GPIO_NR(2, 5)
#define TM_READY_GPIO				IMX_GPIO_NR(2, 6)
#define HP_DET_GPIO					IMX_GPIO_NR(2, 7)
#define BL_ERROR_GPIO				IMX_GPIO_NR(2, 8)
#define X_ASSERT_MIPI_TO_EDP_GPIO	IMX_GPIO_NR(2, 9)
#define STBY_DSI_BRG_GPIO			IMX_GPIO_NR(2, 10)
#define BE_POWER_ON0_GPIO			IMX_GPIO_NR(2, 11)
#define PANEL_DRV_MUTE_GPIO			IMX_GPIO_NR(2, 12)
#define PANEL_DET_GPIO				IMX_GPIO_NR(2, 13)
#define PCIE_NRST_GPIO				IMX_GPIO_NR(2, 14)
#define IMX_LED_SEL_GPIO			IMX_GPIO_NR(2, 15)
#define IMX_ERROR1_LED_GPIO			IMX_GPIO_NR(2, 16)
#define ECO_ON_GPIO					IMX_GPIO_NR(2, 17)
#define FAN_EN_GPIO					IMX_GPIO_NR(2, 18)
#define IMX_POWER_LED_GPIO			IMX_GPIO_NR(2, 19)
#define COMPLETION_NOTIFY_GPIO		IMX_GPIO_NR(2, 20)
#define EDID_SEL1_GPIO				IMX_GPIO_NR(3, 19)
#define EDID_SEL2_GPIO				IMX_GPIO_NR(3, 20)
#define HDMI1_HPD_GPIO				IMX_GPIO_NR(3, 21)
#define EDID_WP_GPIO				IMX_GPIO_NR(3, 22)
#define HDMI2_HPD_GPIO				IMX_GPIO_NR(3, 23)
#define IMX_SELF_RESET_GPIO			IMX_GPIO_NR(3, 24)
#define RST_DSI_LVDS_BRG_GPIO		IMX_GPIO_NR(4, 6)
/*#define USB_BUS_SEL_GPIO			IMX_GPIO_NR(4, 8)*/ /*conflict? X_ETHER_PHY_RESET*/
#define BE_WDT_GPIO					IMX_GPIO_NR(4, 9)
#define X_AUDIO_DSP_RST_GPIO		IMX_GPIO_NR(4, 17)
#define X_AU_MUTE_A_GPIO			IMX_GPIO_NR(4, 18)
#define USB_HUB_RST_GPIO			IMX_GPIO_NR(4, 19)
#define IMX_LAMP_LED_GPIO			IMX_GPIO_NR(4, 23)
#define MAIN_PWR_1_DET_GPIO			IMX_GPIO_NR(4, 24)
#define OCP_VBUS_IMX_GPIO			IMX_GPIO_NR(4, 25)
#define LD_ERROR_GPIO				IMX_GPIO_NR(4, 26)
#define BE_UPDATE_SW_GPIO			IMX_GPIO_NR(5, 2)
#define IMX_STATUS_LED_GPIO			IMX_GPIO_NR(1, 14)

#define CEC_IN_GPIO					IMX_GPIO_NR(4, 28)
#define BOOT_DONE_USB_HUB_GPIO		IMX_GPIO_NR(4, 29)
#define STBY_REBOOT_GPIO			IMX_GPIO_NR(4, 30)

#define CEC_OUT_GPIO				IMX_GPIO_NR(5, 1)

#define IMX_USB_OTG_DISABLE_GPIO	IMX_GPIO_NR(4, 20)
#define FAN_PROT_GPIO				IMX_GPIO_NR(4, 1)
#define BOOT_DONE_DSP_GPIO			IMX_GPIO_NR(4, 2)
#define MAIN_VIDEO_MUTE2_IN_GPIO	IMX_GPIO_NR(4, 3)
#define SUB_VIDEO_MUTE2_IN_GPIO		IMX_GPIO_NR(4, 4)

#define AU_ERROR_GPIO				IMX_GPIO_NR(4, 7)

#define W_PROT_GPIO					IMX_GPIO_NR(4, 15)

#define R2_BOOT_DONE_GPIO			IMX_GPIO_NR(5, 5)
#define IMX_BOOT_DONE_GPIO			IMX_GPIO_NR(5, 4)
#define X_AUDIO_AMP_RST_GPIO		IMX_GPIO_NR(5, 3)

static void setup_gpio(void)
{

	debug("start GPIO init.\n");

	imx_iomux_v3_setup_multiple_pads(gpio_pads, ARRAY_SIZE(gpio_pads));

#define GPIO_REQUEST_IN(x) do{gpio_request(x, #x); gpio_direction_input(x);}while(0)
#define GPIO_REQUEST_OUT(x, y) do{gpio_request(x, #x); gpio_direction_output(x, y);}while(0)

	GPIO_REQUEST_OUT(USB_B_C_SEL_GPIO, 0);

	GPIO_REQUEST_IN(PMIC_XINT_GPIO);
	GPIO_REQUEST_IN(FE_CHG_STATUS_GPIO);
	GPIO_REQUEST_IN(FE_RESUME_IMX_GPIO);
	GPIO_REQUEST_OUT(FE_UPDATE_SW_GPIO, 0);
	GPIO_REQUEST_IN(BOOT_DONE_USB_C_GPIO);
	GPIO_REQUEST_OUT(PWR_RESET_GPIO, 0);
	GPIO_REQUEST_OUT(X_BE_RESET_GPIO, 1);
	GPIO_REQUEST_OUT(LED_ILLUMI_POW_EN_GPIO, 0);
	GPIO_REQUEST_OUT(NOTIFY_FROM_IMX_GPIO, 0);
	GPIO_REQUEST_OUT(SPI_SEL_GPIO, 0);
	GPIO_REQUEST_IN(INT_MIPI_TO_EDP_GPIO);
	GPIO_REQUEST_OUT(USB_UPDATE_EN_GPIO, 0);
	GPIO_REQUEST_OUT(SPI_NR_SEL_GPIO, 0);
	GPIO_REQUEST_OUT(BE_POWER_ON2_GPIO, 0);
	GPIO_REQUEST_OUT(BE_POWER_ON1_GPIO, 0);
	GPIO_REQUEST_IN(TM_READY_GPIO);
	GPIO_REQUEST_IN(HP_DET_GPIO);
	GPIO_REQUEST_IN(BL_ERROR_GPIO);
	GPIO_REQUEST_OUT(X_ASSERT_MIPI_TO_EDP_GPIO, 1 /*0*/); /*tempolary force deassert reset. use power-on-reset instead.*/
	GPIO_REQUEST_OUT(STBY_DSI_BRG_GPIO, 1 /*0*/); /*tempolary force deassert reset. use power-on-reset instead.*/
	GPIO_REQUEST_OUT(BE_POWER_ON0_GPIO, 0);
	GPIO_REQUEST_OUT(PANEL_DRV_MUTE_GPIO, 0);
	GPIO_REQUEST_IN(PANEL_DET_GPIO);
	GPIO_REQUEST_OUT(PCIE_NRST_GPIO, 0);
	GPIO_REQUEST_OUT(IMX_LED_SEL_GPIO, 0);
	GPIO_REQUEST_OUT(IMX_ERROR1_LED_GPIO, 0);
	GPIO_REQUEST_OUT(ECO_ON_GPIO, 0);
	GPIO_REQUEST_OUT(FAN_EN_GPIO,0);
	GPIO_REQUEST_OUT(IMX_POWER_LED_GPIO, 0);
	GPIO_REQUEST_IN(COMPLETION_NOTIFY_GPIO);
	GPIO_REQUEST_OUT(EDID_SEL1_GPIO, 1);
	GPIO_REQUEST_OUT(EDID_SEL2_GPIO, 1);
	GPIO_REQUEST_OUT(HDMI1_HPD_GPIO,0);
	GPIO_REQUEST_OUT(EDID_WP_GPIO, 1);
	GPIO_REQUEST_OUT(HDMI2_HPD_GPIO, 0);
	GPIO_REQUEST_OUT(IMX_SELF_RESET_GPIO, 0);
	GPIO_REQUEST_OUT(RST_DSI_LVDS_BRG_GPIO, 1 /*0*/); /*tempolary force deassert reset. use power-on-reset instead.*/
	/*GPIO_REQUEST_OUT(USB_BUS_SEL_GPIO, 0);*/ /*conflict? X_ETHER_PHY_}RESET*/
	GPIO_REQUEST_IN(BE_WDT_GPIO);
	GPIO_REQUEST_OUT(X_AUDIO_DSP_RST_GPIO, 0);
	GPIO_REQUEST_OUT(X_AU_MUTE_A_GPIO, 0);
	GPIO_REQUEST_OUT(USB_HUB_RST_GPIO, 0);
	GPIO_REQUEST_OUT(IMX_LAMP_LED_GPIO, 0);
	GPIO_REQUEST_IN(MAIN_PWR_1_DET_GPIO);
	GPIO_REQUEST_IN(OCP_VBUS_IMX_GPIO);
	GPIO_REQUEST_IN(LD_ERROR_GPIO);
	GPIO_REQUEST_OUT(BE_UPDATE_SW_GPIO, 0);
	GPIO_REQUEST_OUT(IMX_STATUS_LED_GPIO, 0);

	GPIO_REQUEST_IN(CEC_IN_GPIO);
	GPIO_REQUEST_IN(BOOT_DONE_USB_HUB_GPIO);
	GPIO_REQUEST_OUT(STBY_REBOOT_GPIO, 0);

	GPIO_REQUEST_OUT(CEC_OUT_GPIO, 0);

	GPIO_REQUEST_OUT(IMX_USB_OTG_DISABLE_GPIO, 0);
	GPIO_REQUEST_IN(FAN_PROT_GPIO);
	GPIO_REQUEST_IN(BOOT_DONE_DSP_GPIO);
	GPIO_REQUEST_OUT(MAIN_VIDEO_MUTE2_IN_GPIO, 0);
	GPIO_REQUEST_OUT(SUB_VIDEO_MUTE2_IN_GPIO, 1);

	GPIO_REQUEST_IN(AU_ERROR_GPIO);

	GPIO_REQUEST_IN(W_PROT_GPIO);

	GPIO_REQUEST_IN(R2_BOOT_DONE_GPIO);
	GPIO_REQUEST_OUT(IMX_BOOT_DONE_GPIO, 0);
	GPIO_REQUEST_OUT(X_AUDIO_AMP_RST_GPIO, 0);

	debug("end GPIO init.\n");

/*
	GPIO-HOG can't work on imx8mm?
	always "Header Tag is not an IMX image"
*/
/*
	struct gpio_desc *desc;
	int ret, i;
	static char* pin_names[] = {
		"fe_chg_status",
		"fe_update_sw",
		"bootup_note",
		"pwr_restart",
		"x_be_reset",
		"stby_reboot",
		"edlc_charge_start",
		"spi_sel",
		"pwr_good_pif",
		"rst_pif_n",
		"spi_nr_sel",
		"be_power_on2",
		"be_power_on1",
		"pwr_good_dif",
		"reserved_2_7",
		"irq_dif_n",
		"rst_dif_n",
		"be_power_on0",
		"panel_drv_mute",
		"irq_pif_n",
		"pcie_nrst",
		"led_sel",
		"imx_stby_led",
		"clkgen_update",
		"clkgen_rom_sel",
		"imx_power_led",
		"fe_panel_drv_mute_in",
		"edid_sel1",
		"edid_sel2",
		"hdmi1_hpd",
		"edid_wp",
		"hdmi2_hpd",
		"self_reset",
		"fe_chg_status_m4",
		"rst_dsi_lvds_brg",
		"be_wdt",
		"fan_prot",
		"au_mute_a",
		"pwr_on_vbus_imx",
		"imx_lamp_led",
		"ac_off_det",
		"ocp_vbus_imx",
		"hw_prot_det",
		"cec_in",
		"cec_out",
		"be_update_sw",
	};
	printf("start GPIO init.\n");
	ret = gpio_hog_probe_all();
	if(ret){
		printf("gpio_hog_probe_all error:%d\n", ret);
	}
	for(i = 0; i < ARRAY_SIZE(pin_names); ++i){
		ret = gpio_hog_lookup_name(pin_names[i], &desc);
		if(ret){
			printf("gpio-hog %d:%s not found.\n", i, pin_names[i]);
		}
	}
	printf("end GPIO init %d.\n", i);
*/
}

int board_early_init_f(void)
{
	struct wdog_regs *wdog = (struct wdog_regs *)WDOG1_BASE_ADDR;

	imx_iomux_v3_setup_multiple_pads(wdog_pads, ARRAY_SIZE(wdog_pads));

	set_wdog_reset(wdog);

	imx_iomux_v3_setup_multiple_pads(uart_pads, ARRAY_SIZE(uart_pads));
	/*__raw_writel(3, IOMUXC_BASE_ADDR + 0x4f4 );*/ /*as IOMUXC_UART1_RXD_SELECT_INPUT*/

	init_uart_clk(0);

#ifdef CONFIG_NAND_MXS
	setup_gpmi_nand(); /* SPL will call the board_early_init_f */
#endif

	setup_gpio();

	return 0;
}

#if IS_ENABLED(CONFIG_FEC_MXC)
static int setup_fec(void)
{
	debug("stella2 setup_fec() start\n");

	struct iomuxc_gpr_base_regs *gpr =
		(struct iomuxc_gpr_base_regs *)IOMUXC_GPR_BASE_ADDR;




	/* Use 50MHz?*/
	setbits_le32(&gpr->gpr[1],
			IOMUXC_GPR_GPR1_GPR_ENET1_TX_CLK_SEL_MASK);
	debug("stella2 setup_fec() 25MHz end\n");
	return set_clk_enet(ENET_25MHZ);

	/* Use 125M anatop REF_CLK1 for ENET1, not from external */
/*	clrsetbits_le32(&gpr->gpr[1],
			IOMUXC_GPR_GPR1_GPR_ENET1_TX_CLK_SEL_MASK, 0);

	debug("stella2 setup_fec() 125MHz end\n");

	return set_clk_enet(ENET_125MHZ);*/
}

int board_phy_config(struct phy_device *phydev)
{
	debug("stella2 board_phy_config() start with %p\n", phydev);

	/* enable rgmii rxc skew and phy mode select to RGMII copper */
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1d, 0x1f);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1e, 0x8);

	phy_write(phydev, MDIO_DEVAD_NONE, 0x1d, 0x00);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1e, 0x82ee);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1d, 0x05);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1e, 0x100);

	if (phydev->drv->config)
		phydev->drv->config(phydev);

	debug("stella2 board_phy_config() end with %p\n", (phydev->drv->config));

	return 0;
}
#endif

#ifdef CONFIG_USB_TCPC
struct tcpc_port port1;
struct tcpc_port port2;

static int setup_pd_switch(uint8_t i2c_bus, uint8_t addr)
{
	struct udevice *bus;
	struct udevice *i2c_dev = NULL;
	int ret;
	uint8_t valb;

	ret = uclass_get_device_by_seq(UCLASS_I2C, i2c_bus, &bus);
	if (ret) {
		printf("%s: Can't find bus\n", __func__);
		return -EINVAL;
	}

	ret = dm_i2c_probe(bus, addr, 0, &i2c_dev);
	if (ret) {
		printf("%s: Can't find device id=0x%x\n",
			__func__, addr);
		return -ENODEV;
	}

	ret = dm_i2c_read(i2c_dev, 0xB, &valb, 1);
	if (ret) {
		printf("%s dm_i2c_read failed, err %d\n", __func__, ret);
		return -EIO;
	}
	valb |= 0x4; /* Set DB_EXIT to exit dead battery mode */
	ret = dm_i2c_write(i2c_dev, 0xB, (const uint8_t *)&valb, 1);
	if (ret) {
		printf("%s dm_i2c_write failed, err %d\n", __func__, ret);
		return -EIO;
	}

	/* Set OVP threshold to 23V */
	valb = 0x6;
	ret = dm_i2c_write(i2c_dev, 0x8, (const uint8_t *)&valb, 1);
	if (ret) {
		printf("%s dm_i2c_write failed, err %d\n", __func__, ret);
		return -EIO;
	}

	return 0;
}

int pd_switch_snk_enable(struct tcpc_port *port)
{
	if (port == &port1) {
		debug("Setup pd switch on port 1\n");
		return setup_pd_switch(1, 0x72);
	} else if (port == &port2) {
		debug("Setup pd switch on port 2\n");
		return setup_pd_switch(1, 0x73);
	} else
		return -EINVAL;
}

struct tcpc_port_config port1_config = {
	.i2c_bus = 1, /*i2c2*/
	.addr = 0x50,
	.port_type = TYPEC_PORT_UFP,
	.max_snk_mv = 5000,
	.max_snk_ma = 3000,
	.max_snk_mw = 40000,
	.op_snk_mv = 9000,
	.switch_setup_func = &pd_switch_snk_enable,
};

struct tcpc_port_config port2_config = {
	.i2c_bus = 1, /*i2c2*/
	.addr = 0x52,
	.port_type = TYPEC_PORT_UFP,
	.max_snk_mv = 9000,
	.max_snk_ma = 3000,
	.max_snk_mw = 40000,
	.op_snk_mv = 9000,
	.switch_setup_func = &pd_switch_snk_enable,
};

static int setup_typec(void)
{
	int ret;

	debug("tcpc_init port 2\n");
	ret = tcpc_init(&port2, port2_config, NULL);
	if (ret) {
		printf("%s: tcpc port2 init failed, err=%d\n",
		       __func__, ret);
	} else if (tcpc_pd_sink_check_charging(&port2)) {
		/* Disable PD for USB1, since USB2 has priority */
		port1_config.disable_pd = true;
		printf("Power supply on USB2\n");
	}

	debug("tcpc_init port 1\n");
	ret = tcpc_init(&port1, port1_config, NULL);
	if (ret) {
		printf("%s: tcpc port1 init failed, err=%d\n",
		       __func__, ret);
	} else {
		if (!port1_config.disable_pd)
			printf("Power supply on USB1\n");
		return ret;
	}

	return ret;
}

#endif /* #ifdef CONFIG_USB_TCPC */

static int is_usb_initialized = 0;

int board_usb_init(int index, enum usb_init_type init)
{
	int ret = 0;
#ifdef CONFIG_USB_TCPC
	struct tcpc_port *port_ptr;
#endif /* #ifdef CONFIG_USB_TCPC */

	debug("board_usb_init %d, type %d\n", index, init);
	printf("board_usb_init %d, type %d\n", index, init);

	imx8m_usb_power(index, true);

	switch(index){
	case 0:
		imx_iomux_v3_setup_multiple_pads(usbotg1_pads, ARRAY_SIZE(usbotg1_pads));
		break;
	case 1:
		imx_iomux_v3_setup_multiple_pads(usbotg2_pads, ARRAY_SIZE(usbotg2_pads));
		break;
	default:
		printf("No USB %d\n", index);
		break;
	}

#ifdef CONFIG_USB_TCPC
	if (index == 0)
		port_ptr = &port1;
	else
		port_ptr = &port2;

	if (init == USB_INIT_HOST)
		tcpc_setup_dfp_mode(port_ptr);
	else
		tcpc_setup_ufp_mode(port_ptr);
#endif /* #ifdef CONFIG_USB_TCPC */

	is_usb_initialized = 1;
	return ret;
}

int board_usb_cleanup(int index, enum usb_init_type init)
{
	int ret = 0;

	debug("board_usb_cleanup %d, type %d\n", index, init);

#ifdef CONFIG_USB_TCPC
	if (init == USB_INIT_HOST) {
		if (index == 0)
			ret = tcpc_disable_src_vbus(&port1);
		else
			ret = tcpc_disable_src_vbus(&port2);
	}
#endif /* #ifdef CONFIG_USB_TCPC */

	imx8m_usb_power(index, false);
	return ret;
}

#ifdef CONFIG_USB_TCPC
int board_ehci_usb_phy_mode(struct udevice *dev)
{
	int ret = 0;
	enum typec_cc_polarity pol;
	enum typec_cc_state state;
	struct tcpc_port *port_ptr;

	if (dev->req_seq == 0)
		port_ptr = &port1;
	else
		port_ptr = &port2;

	tcpc_setup_ufp_mode(port_ptr);

	ret = tcpc_get_cc_status(port_ptr, &pol, &state);
	if (!ret) {
		if (state == TYPEC_STATE_SRC_RD_RA || state == TYPEC_STATE_SRC_RD)
			return USB_INIT_HOST;
	}

	return USB_INIT_DEVICE;
}
#endif /* #ifdef CONFIG_USB_TCPC */



static struct udevice *g_bus = NULL;
static struct udevice *g_i2c_dev = NULL;

int board_mt9810_setup(void){
	u32 counter = 0;
	int value = 0;
	int ret = uclass_get_device_by_seq(UCLASS_I2C, 2, &g_bus);
	struct i2c_msg msg;
	if (ret) {
		printf("%s: Can't find bus i2c-2\n", __func__);
		return -EINVAL;
	}

	ret = dm_i2c_probe(g_bus, 0xb2 >> 1, 0, &g_i2c_dev);
	if (ret) {
		printf("%s: Can't find device id=0x%x\n",
			__func__, 0xb2);
		return -ENODEV;
	}

	ret = gpio_request(R2_BOOT_DONE_GPIO, "R2_BOOT_DONE_GPIO");
	if(ret){
		printf("%s: Can't gpio_request R2_BOOT_DONE_GPIO : ret=%d\n", __func__, ret);
	}
	value = gpio_get_value(R2_BOOT_DONE_GPIO);
	/*wait busy released*/
	while(value == 0){
		udelay(100);
		value = gpio_get_value(R2_BOOT_DONE_GPIO);
		counter++;
		if(counter > 100000 /*as 10sec*/){
			printf("%s: R2_BOOT_DONE_GPIO High check 10s timeout\n", __func__);
			break;
		}
	}
	debug("%s: R2_BOOT_DONE_GPIO check pass: %d / %d count\n", __func__, value, (int)counter);

	/*Enter Internal mode*/
	{
		/*
		Reference: https://dev.azure.com/sonylds/Javelin/_git/stella2-core?path=/Libraries/FETalk/Model/Glue/DeviceDriver/Mt9810Driver.cpp
			DeviceDriverResult Mt9810Driver::enterToInternalMode(const ThreadLockObj& lock) {
		*/
		u8 data0_ar[1] = { 0x00 };
		u8 data1_ar[5] = { 0x53, 0x45, 0x52, 0x44, 0x42 };
		u8 data2_ar[1] = { 0x80 };
		u8 data3_ar[1] = { 0x82 };
		u8 data4_ar[1] = { 0x85 };
		u8 data5_ar[1] = { 0x53 };
		u8 data6_ar[1] = { 0x7f };
		u8 data7_ar[1] = { 0x35 };
		u8 data8_ar[1] = { 0x71 };

		/*p_if_i2c_->read(lock, SLAVE_ADDRESS, I2C_SUB_TYPE, 0x00, 0, false);*/
		msg.addr = 0xb2 >> 1;
		msg.flags = I2C_M_STOP | I2C_M_RD;
		msg.len = 0;
		msg.buf = data0_ar;
		ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
		if(ret){
			printf("%s: dm_i2c_xfer data0_ar failed : ret=%d\n", __func__, ret);
		}

		/*result = p_if_i2c_->write(lock, SLAVE_ADDRESS, I2C_SUB_TYPE, 0x00, std::vector<uint8_t>{0x53, 0x45, 0x52, 0x44, 0x42});*/
		msg.addr = 0xb2 >> 1;
		msg.flags = I2C_M_STOP;
		msg.len = 5;
		msg.buf = data1_ar;
		ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
		if(ret){
			printf("%s: dm_i2c_xfer data1_ar failed : ret=%d\n", __func__, ret);
		}

		/*result = p_if_i2c_->write(lock, SLAVE_ADDRESS, I2C_SUB_TYPE, 0x00, std::vector<uint8_t>{0x80});*/
		msg.addr = 0xb2 >> 1;
		msg.flags = I2C_M_STOP;
		msg.len = 1;
		msg.buf = data2_ar;
		ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
		if(ret){
			printf("%s: dm_i2c_xfer data2_ar failed : ret=%d\n", __func__, ret);
		}
		/*result = p_if_i2c_->write(lock, SLAVE_ADDRESS, I2C_SUB_TYPE, 0x00, std::vector<uint8_t>{0x82});*/
		msg.addr = 0xb2 >> 1;
		msg.flags = I2C_M_STOP;
		msg.len = 1;
		msg.buf = data3_ar;
		ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
		if(ret){
			printf("%s: dm_i2c_xfer data3_ar failed : ret=%d\n", __func__, ret);
		}
		/*result = p_if_i2c_->write(lock, SLAVE_ADDRESS, I2C_SUB_TYPE, 0x00, std::vector<uint8_t>{0x85});*/
		msg.addr = 0xb2 >> 1;
		msg.flags = I2C_M_STOP;
		msg.len = 1;
		msg.buf = data4_ar;
		ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
		if(ret){
			printf("%s: dm_i2c_xfer data4_ar failed : ret=%d\n", __func__, ret);
		}
		/*result = p_if_i2c_->write(lock, SLAVE_ADDRESS, I2C_SUB_TYPE, 0x00, std::vector<uint8_t>{0x53});*/
		msg.addr = 0xb2 >> 1;
		msg.flags = I2C_M_STOP;
		msg.len = 1;
		msg.buf = data5_ar;
		ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
		if(ret){
			printf("%s: dm_i2c_xfer data5_ar failed : ret=%d\n", __func__, ret);
		}
		/*result = p_if_i2c_->write(lock, SLAVE_ADDRESS, I2C_SUB_TYPE, 0x00, std::vector<uint8_t>{0x7F});*/
		msg.addr = 0xb2 >> 1;
		msg.flags = I2C_M_STOP;
		msg.len = 1;
		msg.buf = data6_ar;
		ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
		if(ret){
			printf("%s: dm_i2c_xfer data6_ar failed : ret=%d\n", __func__, ret);
		}
		/*result = p_if_i2c_->write(lock, SLAVE_ADDRESS, I2C_SUB_TYPE, 0x00, std::vector<uint8_t>{0x35});*/
		msg.addr = 0xb2 >> 1;
		msg.flags = I2C_M_STOP;
		msg.len = 1;
		msg.buf = data7_ar;
		ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
		if(ret){
			printf("%s: dm_i2c_xfer data7_ar failed : ret=%d\n", __func__, ret);
		}
		/*result = p_if_i2c_->write(lock, SLAVE_ADDRESS, I2C_SUB_TYPE, 0x00, std::vector<uint8_t>{0x71});*/
		msg.addr = 0xb2 >> 1;
		msg.flags = I2C_M_STOP;
		msg.len = 1;
		msg.buf = data8_ar;
		ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
		if(ret){
			printf("%s: dm_i2c_xfer data8_ar failed : ret=%d\n", __func__, ret);
		}
	}

	ret = gpio_free(R2_BOOT_DONE_GPIO);
	if(ret){
		printf("%s: Can't gpio_free R2_BOOT_DONE_GPIO : ret=%d\n", __func__, ret);
	}

	return 0;
}

int board_mt9810_regwrite1(u16 address, u8 data){
	u32 counter = 0;
	int value = 0;
	int ret = 0;
	u8 data_ar[8 + 1 + 1] = {0x10, 0x00, 0x10, 0x33, 0xa0, 0x1, address >> 8, address & 0xff, 0x00, 0x00};
	u8 data_sum = 0;
	struct i2c_msg msg;

	ret = gpio_request(COMPLETION_NOTIFY_GPIO, "COMPLETION_NOTIFY_GPIO");
	if(ret){
		printf("%s: Can't gpio_request COMPLETION_NOTIFY_GPIO : ret=%d\n", __func__, ret);
	}
	value = gpio_get_value(COMPLETION_NOTIFY_GPIO);
	/*wait busy released*/
	while(value != 0){
		udelay(100);
		value = gpio_get_value(COMPLETION_NOTIFY_GPIO);
		counter++;
		if(counter > 10000 /*as 1sec*/){
			printf("%s: COMPLETION_NOTIFY_GPIO Low check 1s timeout\n", __func__);
			break;
		}
	}
	debug("%s: COMPLETION_NOTIFY_GPIO check pass: %d / %d count\n", __func__, value, (int)counter);

	ret = gpio_free(COMPLETION_NOTIFY_GPIO);
	if(ret){
		printf("%s: Can't gpio_free COMPLETION_NOTIFY_GPIO : ret=%d\n", __func__, ret);
	}

	ret = gpio_request(NOTIFY_FROM_IMX_GPIO, "NOTIFY_FROM_IMX_GPIO");
	if(ret){
		printf("%s: Can't gpio_request COMPLETION_NOTIFY_GPIO : ret=%d\n", __func__, ret);
	}
	gpio_set_value(NOTIFY_FROM_IMX_GPIO, 0);

	data_ar[8] = data;
	data_sum = data_ar[6] ^ data_ar[7] ^ data_ar[8];
	data_ar[9] = (data_sum & 0x7f) | 0x80;
	/*i2c_reg_write(0xb2 >> 1, 0, 0, data_ar, 10);*/
	msg.addr = 0xb2 >> 1;
	msg.flags = I2C_M_STOP;
	msg.len = 10;
	msg.buf = data_ar;
	ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
	if(ret){
		printf("%s: Can't dm_i2c_xfer : ret=%d\n", __func__, ret);
		gpio_set_value(NOTIFY_FROM_IMX_GPIO, 1);
		ret = gpio_free(NOTIFY_FROM_IMX_GPIO);
		if(ret){
			printf("%s: Can't gpio_free in -EIO NOTIFY_FROM_IMX_GPIO : ret=%d\n", __func__, ret);
		}
		return -EIO;
	}

	gpio_set_value(NOTIFY_FROM_IMX_GPIO, 1);
	ret = gpio_free(NOTIFY_FROM_IMX_GPIO);
	if(ret){
		printf("%s: Can't gpio_free NOTIFY_FROM_IMX_GPIO : ret=%d\n", __func__, ret);
	}
	return 0;
}

int board_tiny_setup(void){
	int ret = uclass_get_device_by_seq(UCLASS_I2C, 0, &g_bus);
	if (ret) {
		printf("%s: Can't find bus i2c-0\n", __func__);
		return -EINVAL;
	}

	ret = dm_i2c_probe(g_bus, 0x60 >> 1, 0, &g_i2c_dev);
	if (ret) {
		printf("%s: Can't find device id=0x%x\n",
			__func__, 0xb2);
		return -ENODEV;
	}

	return 0;
}

void request_jav2_logo_gpios(void){
	int ret = 0;
	ret = gpio_request(BE_POWER_ON0_GPIO, "BE_POWER_ON0_GPIO");
	if(ret){
		printf("%s: Can't gpio_request BE_POWER_ON0_GPIO : ret=%d\n", __func__, ret);
	}
	ret = gpio_request(BE_POWER_ON2_GPIO, "BE_POWER_ON2_GPIO");
	if(ret){
		printf("%s: Can't gpio_request BE_POWER_ON2_GPIO : ret=%d\n", __func__, ret);
	}
	ret = gpio_request(IMX_BOOT_DONE_GPIO, "IMX_BOOT_DONE_GPIO");
	if(ret){
		printf("%s: Can't gpio_request IMX_BOOT_DONE_GPIO : ret=%d\n", __func__, ret);
	}
}

void free_jav2_logo_gpios(void){
	int ret = 0;
	ret = gpio_free(IMX_BOOT_DONE_GPIO);
	if(ret){
		printf("%s: Can't gpio_free IMX_BOOT_DONE_GPIO : ret=%d\n", __func__, ret);
	}
	ret = gpio_free(BE_POWER_ON2_GPIO);
	if(ret){
		printf("%s: Can't gpio_free BE_POWER_ON2_GPIO : ret=%d\n", __func__, ret);
	}
	ret = gpio_free(BE_POWER_ON0_GPIO);
	if(ret){
		printf("%s: Can't gpio_free BE_POWER_ON0_GPIO : ret=%d\n", __func__, ret);
	}
}

int board_mt9810_logo_init(bool logo_on, bool power_on, int logo_id){
	debug("stella2 board_mt9810_logo_init(logo:%s, power:%s, id:%d)\n", logo_on ? "Y" : "N", power_on ? "Y" : "N", logo_id);
	request_jav2_logo_gpios();
	if(power_on){
		if(logo_on){
			if(board_mt9810_setup()){
				goto failed;
			}
			switch(logo_id){
			default:
			case 0: /*SONY logo*/
				/*Change fV for Pattern:240Hz*/
				if(board_mt9810_regwrite1(0x210, 3 << 2)){
					goto failed;
				}
			/*Pattern Select*/
				if(board_mt9810_regwrite1(0x228, 7 << 3)){
					goto failed;
				}
			/*Pattern ON*/
				if(board_mt9810_regwrite1(0x228, 7 << 3 | 0x80)){
					goto failed;
				}
				/*Video Mute off*/
				if(board_mt9810_regwrite1(0xa4c, 0 << 0)){
					goto failed;
				}
				break;
			case 1: /*user0*/
			case 2: /*user1*/
			case 3: /*user2*/
			case 4: /*user3*/
				/*Change fV for Pattern:60Hz*/
				if(board_mt9810_regwrite1(0x210, 1 << 2)){
					goto failed;
				}
				/*Picture select*/
				if(board_mt9810_regwrite1(0x226, (logo_id - 1) << 2)){
					goto failed;
				}
				/*Picture Enable*/
				if(board_mt9810_regwrite1(0x229, 1 << 0)){
					goto failed;
				}
				/*Video Mute off*/
				if(board_mt9810_regwrite1(0xa4c, 0 << 0)){
					goto failed;
				}
				break;
			}
			/*BL_ON:HIGH*/
			gpio_set_value(BE_POWER_ON0_GPIO, 1);
			/*TCON_ON:HIGH*/
			gpio_set_value(BE_POWER_ON2_GPIO, 1);

			/*IMX_BOOT_DONE_GPIO:HIGH*/
			gpio_set_value(IMX_BOOT_DONE_GPIO, 1);
		} else {
			/*BL_ON:HIGH*/
			gpio_set_value(BE_POWER_ON0_GPIO, 1);
			/*TCON_ON:HIGH*/
			gpio_set_value(BE_POWER_ON2_GPIO, 1);
		}
	} else {
		/*BL_ON:LOW*/
		gpio_set_value(BE_POWER_ON0_GPIO, 0);
		/*TCON_ON:LOW*/
		gpio_set_value(BE_POWER_ON2_GPIO, 0);
	}
	free_jav2_logo_gpios();
	return 0;
failed:
	free_jav2_logo_gpios();
	return -EIO;
}

int board_mt9810_tiny_lcdv1_logo_init(bool logo_on, bool power_on, int logo_id){
	debug("stella2 board_mt9810_tiny_lcdv1_logo_init(logo:%s, power:%s, id:%d)\n", logo_on ? "Y" : "N", power_on ? "Y" : "N", logo_id);
	request_jav2_logo_gpios();
	if(power_on){
		if(logo_on){
			if(board_mt9810_setup()){
				goto failed;
			}
			switch(logo_id){
			default:
			case 0: /*SONY logo*/
				/*Change fV for Pattern:240Hz*/
				if(board_mt9810_regwrite1(0x210, 3 << 2)){
					goto failed;
				}
			/*Pattern Select*/
				if(board_mt9810_regwrite1(0x228, 7 << 3)){
					goto failed;
				}
			/*Pattern ON*/
				if(board_mt9810_regwrite1(0x228, 7 << 3 | 0x80)){
					goto failed;
				}
				/*Video Mute off*/
				if(board_mt9810_regwrite1(0xa4c, 0 << 0)){
					goto failed;
				}
			case 1: /*user0*/
			case 2: /*user1*/
			case 3: /*user2*/
			case 4: /*user3*/
				/*Change fV for Pattern:60Hz*/
				if(board_mt9810_regwrite1(0x210, 1 << 2)){
					goto failed;
				}
				/*Picture select*/
				if(board_mt9810_regwrite1(0x226, (logo_id - 1) << 2)){
					goto failed;
				}
				/*Picture Enable*/
				if(board_mt9810_regwrite1(0x229, 1 << 0)){
					goto failed;
				}
				/*Video Mute off*/
				if(board_mt9810_regwrite1(0xa4c, 0 << 0)){
					goto failed;
				}
				break;
			}
			/*BL_ON:HIGH*/
			gpio_set_value(BE_POWER_ON0_GPIO, 1);
			/*TCON_ON:HIGH*/
			gpio_set_value(BE_POWER_ON2_GPIO, 1);
			/*FREERUN*/
			if(board_tiny_setup()){
				goto failed;
			}
			{
				static const u8 freerun_ar[7] = {0xc2, 0x02, 0x00, 0x01, 0x00, 0x00, 0x3a};
				struct i2c_msg msg;
				int ret = 0;
				msg.addr = 0x60 >> 1;
				msg.flags = I2C_M_STOP;
				msg.len = 7;
				msg.buf = (u8*)freerun_ar;
				ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
				if(ret){
					goto failed;
				}
				/*i2c_reg_write(0x60 >> 1, 0, 0, freerun_ar, 7);*/
			}
			udelay(10000); /* as 10ms */
			/*FLAT*/
			{
				static const u8 flat_ar[7] = {0xc2, 0x03, 0x00, 0x01, 0x00, 0x00, 0x39};
				struct i2c_msg msg;
				int ret = 0;
				msg.addr = 0x60 >> 1;
				msg.flags = I2C_M_STOP;
				msg.len = 7;
				msg.buf = (u8*)flat_ar;
				ret = dm_i2c_xfer(g_i2c_dev, &msg, 1);
				if(ret){
					goto failed;
				}
				/*i2c_reg_write(0x60 >> 1, 0, 0, flat_ar, 7);*/
			}
			/*IMX_BOOT_DONE_GPIO:HIGH*/
			gpio_set_value(IMX_BOOT_DONE_GPIO, 1);
		} else {
			/*BL_ON:HIGH*/
			gpio_set_value(BE_POWER_ON0_GPIO, 1);
			/*TCON_ON:HIGH*/
			gpio_set_value(BE_POWER_ON2_GPIO, 1);
		}
	} else {
		/*BL_ON:LOW*/
		gpio_set_value(BE_POWER_ON0_GPIO, 0);
		/*TCON_ON:LOW*/
		gpio_set_value(BE_POWER_ON2_GPIO, 0);
	}
	free_jav2_logo_gpios();
	return 0;
failed:
	free_jav2_logo_gpios();
	return -EIO;
}


#define FSL_SIP_GPC			0xC2000000
#define FSL_SIP_CONFIG_GPC_PM_DOMAIN	0x3
#define DISPMIX				9
#define MIPI				10

int board_init(void)
{
	struct arm_smccc_res res;

	debug("stella2 board_init() start\n");
#ifdef CONFIG_USB_TCPC
	setup_typec();
#endif

	if (IS_ENABLED(CONFIG_FEC_MXC))
		setup_fec();

	arm_smccc_smc(IMX_SIP_GPC, IMX_SIP_GPC_PM_DOMAIN,
		      DISPMIX, true, 0, 0, 0, 0, &res);
	arm_smccc_smc(IMX_SIP_GPC, IMX_SIP_GPC_PM_DOMAIN,
		      MIPI, true, 0, 0, 0, 0, &res);

	debug("stella2 board_init() end\n");
	return 0;
}

int board_late_init(void)
{
	bool display_logo_is_on = (env_get_yesno("display_logo") == 1);
	bool display_power_is_on = (env_get_yesno("display_power") == 1);
	int display_panel = env_get_ulong("display_panel", 10, 0);
	char* display_logo_picture = env_get("display_logo_picture");
	int display_logo_id = 0; /*0:SONY logo, 1:BMP logo0, 2:BMP logo1...*/

	debug("stella2 board_late_init() start\n");

#ifdef CONFIG_ENV_IS_IN_MMC
	board_late_mmc_env_init();
#endif

#ifdef CONFIG_ENV_VARS_UBOOT_RUNTIME_CONFIG
	env_set("board_name", "STELLA2 GamingProto");
	env_set("board_rev", "iMX8MM");
#endif
	if(display_logo_picture != NULL){
		if(strncmp("sony", display_logo_picture, 4) == 0){
			display_logo_id = 0;
		} else {
			if(strncmp("user", display_logo_picture, 4) == 0){
				display_logo_id = 1 + *(display_logo_picture + 4) - 0x30 /*as 0*/;
				if(display_logo_id < 0 || display_logo_id > 9){
					display_logo_id = 0;
				}
			}
		}
	} else {
		display_logo_id = 0;
	}

	switch(display_panel){
	default:
		debug("stella2 board_late_init() NO panel assigned(%d)\n", display_panel);
		break;
	case 0: /* as MT9810 / NoPanel */
		board_mt9810_logo_init(display_logo_is_on, display_power_is_on, display_logo_id);
		break;
	case 1: /* as MT9810 / TinyMac / LCDv1 */
		board_mt9810_tiny_lcdv1_logo_init(display_logo_is_on, display_power_is_on, display_logo_id);
		break;
	}

	debug("stella2 board_late_init() end, is_usb_initialized=%d\n", is_usb_initialized);
	return 0;
}

#ifdef CONFIG_FSL_FASTBOOT
#ifdef CONFIG_ANDROID_RECOVERY
int is_recovery_key_pressing(void)
{
	return 0; /*TODO*/
}
#endif /*CONFIG_ANDROID_RECOVERY*/
#endif /*CONFIG_FSL_FASTBOOT*/

#ifdef CONFIG_ANDROID_SUPPORT
bool is_power_key_pressed(void) {
	return (bool)(!!(readl(SNVS_HPSR) & (0x1 << 6)));
}
#endif

